/*
 * Copyright (C) 2012 - Juan Ferrer Toribio
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef SQL_OBJECT_H
#define SQL_OBJECT_H

#include <gvn/gvn.h>

#define SQL_TYPE_OBJECT				(sql_object_get_type ())
#define SQL_OBJECT(obj)				(G_TYPE_CHECK_INSTANCE_CAST (obj, SQL_TYPE_OBJECT, SqlObject))
#define SQL_IS_OBJECT(obj)			(G_TYPE_CHECK_INSTANCE_TYPE (obj, SQL_TYPE_OBJECT))
#define SQL_OBJECT_CLASS(klass)		(G_TYPE_CHECK_CLASS_CAST (klass, SQL_TYPE_OBJECT, SqlObjectClass))
#define SQL_OBJECT_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS (obj, SQL_TYPE_OBJECT, SqlObjectClass))

typedef struct _SqlObject SqlObject;
typedef struct _SqlObjectClass SqlObjectClass;

#include "sql-render.h"

typedef gboolean (* SqlObjectIsReadyFunc) (SqlObject * obj);

struct _SqlObject
{
	GInitiallyUnowned parent;
	gchar * sql;
};

struct _SqlObjectClass
{
	/* <private> */
	GInitiallyUnownedClass parent;
	SqlRenderFunc render;
	SqlObjectIsReadyFunc is_ready;
};

GType		sql_object_get_type		();
void		sql_object_render		(SqlObject * obj, SqlRender * render);
gboolean	sql_object_is_ready		(SqlObject * obj);
void		sql_object_list_add		(SqlObject * obj, GSList ** list, SqlObject * sub);
void		sql_object_list_free	(SqlObject * obj, GSList * list);

#endif
